# Generated by Django 5.1.7 on 2025-04-03 11:42

import aleksis.core.mixins
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0081_migrate_announcements'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RemoveField(
            model_name='announcement',
            name='id',
        ),
        migrations.AlterField(
            model_name='announcement',
            name='calendarevent_ptr',
            field=models.OneToOneField(auto_created=True, null=False, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.calendarevent'),
        ),
        migrations.RemoveField(
            model_name='announcement',
            name='extended_data',
        ),
        migrations.RemoveField(
            model_name='announcement',
            name='managed_by_app_label',
        ),
        migrations.RemoveField(
            model_name='announcement',
            name='valid_from',
        ),
        migrations.RemoveField(
            model_name='announcement',
            name='valid_until',
        ),
    ]
