mutation requestPasswordReset($email: String!) {
  requestPasswordReset(email: $email) {
    success
  }
}

query passwordRequestKeyVerificationStatus($key: String!) {
  passwordRequestKeyVerificationStatus(key: $key)
}

mutation resetPassword($key: String!, $password: String!) {
  resetPassword: changeUserPassword(password: $password, resetKey: $key) {
    success
  }
}
