query gqlSystemProperties {
  systemProperties {
    availableLanguages {
      code
      nameTranslated
      nameLocal
      cookie
    }
    defaultLanguage {
      code
      nameTranslated
      nameLocal
      cookie
    }
    sitePreferences {
      themePrimary
      themeSecondary
      themeLogo {
        url
      }
      generalTitle
      accountPersonPreferPhoto
      footerImprintUrl
      footerPrivacyUrl
      inviteEnabled
      signupEnabled
    }
  }
}

query gqlUsernameSystemProperties {
  usernameSystemProperties: systemProperties {
    sitePreferences {
      authAllowedUsernameRegex
      authDisallowedUids
    }
  }
}
