# Generated by Django 4.2.6 on 2023-10-11 11:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0075_address_seperate_model"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="person",
            name="housenumber",
        ),
        migrations.RemoveField(
            model_name="person",
            name="place",
        ),
        migrations.RemoveField(
            model_name="person",
            name="postal_code",
        ),
        migrations.RemoveField(
            model_name="person",
            name="street",
        ),
    ]
