{% extends 'core/base.html' %}
{% load i18n %}
{% block auth_content %}
  {% block inner_content %}
  {% endblock %}
{% endblock %}
{% block browser_title %}
  {% block head_title %}
  {% endblock head_title %}
{% endblock %}
