# Generated by Django 5.1.3 on 2024-11-15 18:12

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0067_rrule_until'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='calendarevent',
            constraint=models.UniqueConstraint(condition=models.Q(('amends__isnull', False), ('datetime_start__isnull', False)), fields=('datetime_start', 'datetime_end', 'amends'), name='unique_calendar_event_per_amends_and_datetimes'),
        ),
        migrations.AddConstraint(
            model_name='calendarevent',
            constraint=models.UniqueConstraint(condition=models.Q(('amends__isnull', False), ('date_start__isnull', False)), fields=('date_start', 'date_end', 'amends'), name='unique_calendar_event_per_amends_and_dates'),
        ),
    ]
