# Generated by Django 5.1.5 on 2025-04-01 13:16

import aleksis.core.mixins
import django.contrib.postgres.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0085_restructure_dashboard'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CalendarWidget',
            fields=[
                ('dashboardwidget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.dashboardwidget')),
                ('selected_calendars', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(choices=[], max_length=255), blank=True, default=[], help_text='If no specific calendars are selected, all calendars are allowed.', size=None, verbose_name='Allowed calendars')),
            ],
            options={
                'verbose_name': 'Calendar widget',
                'verbose_name_plural': 'Calendar widgets',
            },
            bases=('core.dashboardwidget',),
        ),
        migrations.CreateModel(
            name='CalendarWidgetConfiguration',
            fields=[
                ('dashboardwidgetconfiguration_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.dashboardwidgetconfiguration')),
                ('selected_calendars', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(choices=[], max_length=255), blank=True, default=[], size=None, verbose_name='Selected calendars')),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('core.dashboardwidgetconfiguration',),
        ),
    ]
