# Generated by Django 5.1.1 on 2024-10-13 17:14

import django.contrib.postgres.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0065_calendarevent_update_timezone_constraints'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='grouptype',
            name='owners_can_see_groups',
            field=models.BooleanField(default=False, verbose_name='Owners of groups with this group type can see the groups'),
        ),
        migrations.AddField(
            model_name='grouptype',
            name='owners_can_see_members',
            field=models.BooleanField(default=False, verbose_name='Owners of groups with this group type can see group members'),
        ),
        migrations.AddField(
            model_name='grouptype',
            name='owners_can_see_members_allowed_information',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, choices=[('personal_details', 'Personal details'), ('address', 'Address'), ('contact_details', 'Contact details'), ('photo', 'Photo'), ('avatar', 'Avatar'), ('groups', 'Groups')], max_length=255), blank=True, default=[], size=None, verbose_name="Information owners of groups with this group type can see of the group's members"),
        ),
    ]
